import logging
import uvicorn
import os
from api import app

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def run_api() -> None:
    """Run the FastAPI application."""
    if "james" in os.environ.get("USER", ""):
        logger.info("Running in James's environment")
        uvicorn.run("api:app", host="0.0.0.0", port=8080, reload=True)
    else:
        uvicorn.run(app, host="0.0.0.0", port=8080)


if __name__ == "__main__":
    run_api()


import os
from pydantic_settings import BaseSettings

class ClickUpSettings(BaseSettings):
    clickup_api_key: str
    clickup_team_id: str

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


class LocalSettings(BaseSettings):
    research_output_dir: str = "research output"  # phase 2.
    question_output_dir: str = "question output"  # phase 3. -> phase 4.
    audio_output_dir: str = "audio output"  # phase 4. -> phase 5.
    transcripts_output_dir: str = "transcripts output"  # phase 5. -> phase 6.
    challenge_output_dir: str = "challenge output"  # phase 6. -> phase 7.
    content_generation_dir: str = "content generation output"  # phase 7. -> phase 8.
    review_output_dir: str = "review output"  # phase 8. -> phase 9.


from pydantic import BaseModel

class InitializeRagRequest(BaseModel):
    chat_model: str = "gemma3:1b"
    embedding_model: str = "nomic-embed-text:latest"

class TextRequest(BaseModel):
    text: str
    task_id: str | None = None
    chat_model: str = "gemma3:1b"
    embedding_model: str = "nomic-embed-text:latest"


class RagRequest(BaseModel):
    chat_model: str = "gemma3:1b"
    embedding_model: str = "nomic-embed-text:latest"
    question: str
    mode: str = "local"
    top_k: int = 5
    similarity_threshold: float = 0.0
    max_tokens: int = 2048


import os
from crawl4ai.markdown_generation_strategy import DefaultMarkdownGenerator
from crawl4ai import AsyncWebCrawler, BrowserConfig, CacheMode, CrawlerRunConfig
import subprocess
import requests
from datetime import date, datetime, timedelta
import platform
import asyncio
from playwright.async_api import async_playwright

from settings import ClickUpSettings, LocalSettings

if platform.system() == "Darwin":
    # macOS: use whisper-mps
    from whisper_mps.whisper.transcribe import transcribe as whisper_transcribe

    def transcribe_audio(file_path: str):
        """Transcribe audio file to text using whisper_mps."""
        result = whisper_transcribe(file_path, model="small")
        return result["text"]

else:
    # Linux/other: use faster-whisper
    from faster_whisper import WhisperModel

    def transcribe_audio(file_path: str):
        """Transcribe audio file to text using faster-whisper."""
        model = WhisperModel("small")
        segments, info = model.transcribe(file_path)
        # Concatenate all segments
        return " ".join([segment.text for segment in segments])


fit_md_generator = DefaultMarkdownGenerator(
    content_source="fit_html", options={"ignore_links": True}
)


def get_ollama_host():
    if platform.system() == "Darwin":
        print("Using localhost for Ollama API")
        return "http://localhost:11434"
    else:
        print("Using Docker container for Ollama API")
        return "http://ollama:11434"

def get_ollama_models():
    """Fetch available models from Ollama API."""
    url = f"{get_ollama_host()}/api/tags"
    response = requests.get(url)
    response.raise_for_status()
    return response.json().get("models", [])

def get_ollama_questions(text: str, model: str = "gemma3:1b"):
    """Generate a list of questions based on the prompt text using Ollama API."""
    url = f"{get_ollama_host()}/v1/chat/completions"
    payload = {
        "model": model,
        "messages": [
            {
                "role": "system",
                "content": f"""
                You are an interviewer tasked with creating thoughtful, reusable interview questions on the topic of Dungeons and dragons. 
                Your questions must follow these five parameters:
                1. Breadth - Cover the six journalist anchors (Who, What, When, Where, Why, How).
                2. Creator-Centric - Include process, behind-the-scenes, origin/inspiration, and insights/lessons learned.
                3. Relevance - Keep every question tied to the overarching topic, so answers are useful across multiple content formats (blog, posts, newsletter, Patreon, VIP).
                4. Tangibility - Each question should have a mirrored version that asks, "So what? What's in it for the reader, client, fan, or peer?"
                5. Business Transferability - Include at least one variant of each question that shifts the focus from "Here's my journey" to "Here's a lesson others could apply in their own business or creative practice."
                Output your questions organised by category (Who, What, When, Where, Why, How). For each question, include:
                •  The creator-centric version
                •  The tangible/client-centric version
                •  The business transferability version.
                """,
            },
            {
                "role": "system",
                "content": "Return a list of questions in formatted markdown.",
            },
            {
                "role": "system",
                "content": "Only respond with the markdown content, no explanations.",
            },
            {"role": "user", "content": text},
        ],
    }
    response = requests.post(url, json=payload)
    response.raise_for_status()
    generated_text = response.json()["choices"][0]["message"]["content"]

    # Remove markdown code block markers if present
    if generated_text.startswith("```html"):
        generated_text = generated_text[len("```html") :].strip()
    if generated_text.endswith("```"):
        generated_text = generated_text[:-3].strip()
    return generated_text


def get_ollama_summary(text: str, model: str = "gemma3:1b"):
    """Get a summary of the text using Ollama API."""
    url = f"{get_ollama_host()}/v1/chat/completions"
    payload = {
        "model": model,
        "messages": [
            {
                "role": "system",
                "content": "You are a helpful assistant that summarizes text.",
            },
            {
                "role": "system",
                "content": "Please return the output in formatted html.",
            },
            {
                "role": "system",
                "content": "Use HTML tags like <p>, <b>, <i>, <ul>, <li> for formatting.",
            },
            {"role": "system", "content": "Use nice css styles to make it look cool."},
            {
                "role": "system",
                "content": "Only respond with the HTML content, no explanations.",
            },
            {"role": "user", "content": text},
        ],
    }
    response = requests.post(url, json=payload)
    response.raise_for_status()
    generated_text = response.json()["choices"][0]["message"]["content"]

    # Remove markdown code block markers if present
    if generated_text.startswith("```html"):
        generated_text = generated_text[len("```html") :].strip()
    if generated_text.endswith("```"):
        generated_text = generated_text[:-3].strip()
    return generated_text


def internal_get_ollama_summary(text: str, model: str = "gemma3:1b"):
    """Get a summary of the text using Ollama API."""
    url = f"{get_ollama_host()}/v1/chat/completions"
    payload = {
        "model": model,
        "messages": [
            {
                "role": "system",
                "content": "You are a helpful assistant that summarizes text.",
            },
            {
                "role": "system",
                "content": "Please return max 10 sentences summary in plain text.",
            },
            {
                "role": "system",
                "content": "Only summarize, do not add comments, or flavor text.",
            },
            {"role": "user", "content": text},
        ],
    }
    response = requests.post(url, json=payload)
    response.raise_for_status()
    generated_text = response.json()["choices"][0]["message"]["content"]

    return generated_text


def get_ollama_transcript_cleanup(text: str, model: str = "gemma3:1b", system_prompt: str = ""):
    """Clean up a transcript using Ollama API."""
    url = f"{get_ollama_host()}/v1/chat/completions"
    payload = {
        "model": model,
        "messages": [
            {
                "role": "system",
                "content": system_prompt or "You are a helpful assistant that cleans up transcripts.",
            },
            {"role": "user", "content": text},
        ],
    }
    response = requests.post(url, json=payload)
    response.raise_for_status()
    generated_text = response.json()["choices"][0]["message"]["content"]

    return generated_text


def get_ollama_content_generation(prompt: str, model: str = "gemma3:1b", system_prompt: str = "") -> str:
    """Generate content based on a prompt using Ollama API."""
    url = f"{get_ollama_host()}/v1/chat/completions"
    payload = {
        "model": model,
        "messages": [
            {
                "role": "system",
                "content": system_prompt or "You are a helpful assistant that generates content.",
            },
            {"role": "user", "content": prompt},
        ],
    }
    response = requests.post(url, json=payload)
    response.raise_for_status()
    generated_text = response.json()["choices"][0]["message"]["content"]

    # Remove markdown code block markers if present
    if generated_text.startswith("```markdown"):
        generated_text = generated_text[len("```markdown") :].strip()
    if generated_text.endswith("```"):
        generated_text = generated_text[:-3].strip()
    return generated_text


def convert_webm_to_mp3(webm_path, mp3_path):
    subprocess.run(["ffmpeg", "-y", "-i", webm_path, mp3_path], check=True)


async def get_real_article_url_with_consent(google_news_url):
    """Use Playwright to navigate to the URL and handle consent pages."""

    # get everything after the continue=
    continue_url = google_news_url.split("continue=")[-1]
    print(f"Continue URL: {continue_url}")
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True, slow_mo=3000)
        page = await browser.new_page()
        await page.goto(continue_url, wait_until="domcontentloaded")
        # If consent page appears, click "Accept all" or "Reject all"
        if "consent.google.com" in page.url:
            try:
                # Try to click "Accept all" button
                await page.click("button:has-text('Accept all')")
            except Exception:
                # Try to click "Reject all" if "Accept all" not found
                try:
                    await page.click("button:has-text('Reject all')")
                except Exception:
                    pass
            # Wait for redirect
            await page.wait_for_load_state("domcontentloaded")
        final_url = page.url
        await browser.close()
        return final_url


async def scrape_website_content(url: str) -> str:
    """Scrape website content using crawl4ai."""
    final_url = await get_real_article_url_with_consent(url)
    print(f"Final URL after consent: {final_url}")
    # Use crawl4ai to extract content
    try:
        async with AsyncWebCrawler(
            config=BrowserConfig(
                headless=True,
                enable_stealth=True,  # Enables stealth mode
                user_agent="Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 Chrome/116.0.0.0 Safari/537.36",
            )
        ) as crawler:
            result = await crawler.arun(
                url=final_url,
                config=CrawlerRunConfig(
                    wait_until="domcontentloaded",
                    page_timeout=60000,
                    cache_mode=CacheMode.BYPASS,
                    markdown_generator=fit_md_generator,
                ),
            )
            content = result.markdown
            print(f"Scraped content: {content[:500]}...")  # Print first 500 chars
            # return content
            summarized_content = internal_get_ollama_summary(
                text=content, model="granite3.1-moe:1b"
            )
            return summarized_content, final_url
    except Exception as e:
        print(f"Error fetching {url}: {e}")


def upload_file_from_bytes(filename: str, data: bytes, output_dir: str) -> str:
    """Write bytes to a local file under LocalSettings.research_output_dir.

    Args:
        filename: Name of the file to write.
        data: Bytes content to write.
        mime_type: Optional MIME type (ignored).
        folder_id: Optional folder id (ignored).

    Returns:
        The path to the written file.
    """
    # sanitize filename a bit
    safe_name = filename.replace("/", "_").replace("\\", "_")
    timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    out_path = os.path.join(output_dir, f"{timestamp}_{safe_name}")

    with open(out_path, "wb") as fh:
        fh.write(data)

    return out_path



from fastapi import APIRouter, HTTPException
from lightrag import QueryParam
import os

# local imports
from clickup_utils import fetch_clickup_task, update_clickup_task
from utils import upload_file_from_bytes
from rag_utils import initialize_rag
from schemas import InitializeRagRequest, RagRequest, TextRequest
from settings import LocalSettings

router = APIRouter(prefix="/rag", tags=["rag"])


@router.post("/initialize")
async def initialize_rag_endpoint(request: InitializeRagRequest):
    try:
        rag = await initialize_rag(
            chat_model=request.chat_model, embed_model=request.embedding_model
        )
        return {"message": "RAG initialized successfully"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/query")
async def query_rag_endpoint(request: RagRequest):
    """Query the RAG system with a question and return the answer."""
    try:
        rag = await initialize_rag(
            chat_model=request.chat_model or "gemma3:1b",
            embed_model=request.embedding_model or "nomic-embed-text:latest",
        )
        response = await rag.aquery(
            query=request.question,
            param=QueryParam(mode=request.mode, top_k=request.top_k),
        )
        print("RAG response:", response)
        return {"response": response}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/questions")
async def question_rag_endpoint(request: TextRequest):
    """Query the RAG system with a question and return the answer."""
    try:
        task = fetch_clickup_task(task_id="869ak7hj3")
        system_prompt = task.get("text_content", "") if task else ""
    except Exception as e:
        system_prompt = ""
        print(f"Warning: Failed to fetch ClickUp task {request.task_id}: {e}")

    try:
        rag = await initialize_rag(
            chat_model=request.chat_model or "gemma3:1b",
            embed_model=request.embedding_model or "nomic-embed-text:latest",
        )
        response = await rag.aquery(
            query=request.text,
            param=QueryParam(mode="hybrid", top_k=5),
            system_prompt=system_prompt,
        )
        filename = f"{request.task_id or 'unknown_task'}.txt"
        upload_file_from_bytes(
            filename=filename,
            data=response.encode("utf-8"),
            output_dir=LocalSettings().question_output_dir,
        )
        if request.task_id:
            update_clickup_task(
                task_id=request.task_id,
                status="phase 4. interview",
                description=response,
            )
        return {"detail": f"Question generated and saved for task {request.task_id}"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/reload")
async def reload_rag_endpoint(request: InitializeRagRequest):
    """Scan the RAG working directory, enqueue new text files and process the pipeline."""
    try:
        # initialize RAG and storages
        rag = await initialize_rag(
            chat_model=request.chat_model, embed_model=request.embedding_model
        )

        # determine working dir
        working_dir = getattr(rag, "working_dir", LocalSettings().research_output_dir)

        docs = []
        paths = []
        for root, _, files in os.walk(working_dir):
            for fn in files:
                if fn.lower().endswith((".txt", ".md", ".html", ".htm")):
                    full = os.path.join(root, fn)
                    try:
                        with open(full, "r", encoding="utf-8", errors="ignore") as fh:
                            content = fh.read()
                            if content and content.strip():
                                docs.append(content)
                                paths.append(full)
                    except Exception:
                        continue

        if not docs:
            return {
                "message": "RAG reloaded: no text files found in working_dir",
                "working_dir": working_dir,
            }

        track_id = await rag.apipeline_enqueue_documents(input=docs, file_paths=paths)
        await rag.apipeline_process_enqueue_documents()

        return {
            "message": "RAG reloaded and files enqueued/processed",
            "track_id": track_id,
            "files": len(docs),
        }
    except HTTPException:
        raise
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))



from fastapi import APIRouter, HTTPException
from datetime import datetime

# local imports
from clickup_utils import fetch_clickup_task, update_clickup_task
from schemas import TextRequest
from settings import LocalSettings
from utils import (
    get_ollama_content_generation,
    get_ollama_models,
    get_ollama_summary,
    get_ollama_questions,
    get_ollama_transcript_cleanup,
    scrape_website_content,
    upload_file_from_bytes,
)

router = APIRouter(prefix="/text", tags=["text"])

@router.get("/models")
async def get_models():
    try:
        models = get_ollama_models()
        return {"models": [model.get("name") for model in models]}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

@router.post("/summarize")
async def summarize_text(request: TextRequest):
    try:
        summary = get_ollama_summary(text=request.text, model=request.chat_model)
        return {"summary": summary}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))
    

@router.post("/transcript-cleanup")
async def cleanup_transcript(request: TextRequest):
    try:
        task = fetch_clickup_task(task_id="869akcv3p")
        system_prompt = task.get("text_content", "") if task else ""
    except Exception as e:
        system_prompt = ""
        print(f"Warning: Failed to fetch system prompt from ClickUp task: {e}")
    try:
        print(f"Chosen model: {request.chat_model}")
        cleaned_text = get_ollama_transcript_cleanup(
            text=request.text, model=request.chat_model, system_prompt=system_prompt
        )
        if request.task_id:
            try:
                update_clickup_task(
                    task_id=request.task_id,
                    status="phase 6. transcript clean",
                    description=cleaned_text,
                )
            except Exception as e:
                print(f"Warning: Failed to update ClickUp task {request.task_id}: {e}")

            try:
                file_name = f"cleaned_{request.task_id or 'unknown_task'}.txt"
                upload_file_from_bytes(
                    filename=file_name,
                    data=cleaned_text.encode("utf-8"),
                    output_dir=LocalSettings().transcripts_output_dir,
                )
            except Exception as e:
                print(
                    f"Warning: Failed to upload cleaned transcript for task {request.task_id}: {e}"
                )
        return {"cleaned_text": cleaned_text}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))
    

@router.post("/challenge-ideas")
async def generate_challenge_ideas(request: TextRequest):
    try:
        task = fetch_clickup_task(task_id="869akcv6d")
        system_prompt = task.get("text_content", "") if task else ""
    except Exception as e:
        system_prompt = ""
        print(f"Warning: Failed to fetch system prompt from ClickUp task: {e}")

    try:
        print(f"Chosen model: {request.chat_model}")
        content = get_ollama_content_generation(
            prompt=request.text, model=request.chat_model, system_prompt=system_prompt
        )

        if request.task_id:
            try:
                update_clickup_task(
                    task_id=request.task_id,
                    status="phase 7. challenge",
                    description=content,
                )
            except Exception as e:
                print(f"Warning: Failed to update ClickUp task {request.task_id}: {e}")

            try:
                file_name = f"challenge_ideas_{request.task_id or 'unknown_task'}.txt"
                upload_file_from_bytes(
                    filename=file_name,
                    data=content.encode("utf-8"),
                    output_dir=LocalSettings().content_generation_dir,
                )
            except Exception as e:
                print(
                    f"Warning: Failed to upload generated content for task {request.task_id}: {e}"
                )

        return {"content": content}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/generate-content")
async def generate_content(request: TextRequest):
    try:
        prompt_task = fetch_clickup_task(task_id="869ak6tvx")
        prompt = prompt_task.get("text_content", "") if prompt_task else ""
        style_task = fetch_clickup_task(task_id="869akjejr")
        style = style_task.get("text_content", "") if style_task else ""
        system_prompt = f"{prompt}\n\nStyle Guidelines:\n{style}"
    except Exception as e:
        system_prompt = ""
        print(f"Warning: Failed to fetch system prompt from ClickUp task: {e}")

    try:
        print(f"Chosen model: {request.chat_model}")
        content = get_ollama_content_generation(
            prompt=request.text, model=request.chat_model, system_prompt=system_prompt
        )

        if request.task_id:
            try:
                update_clickup_task(
                    task_id=request.task_id,
                    status="phase 8. draft channel",
                    description=content,
                )
            except Exception as e:
                print(f"Warning: Failed to update ClickUp task {request.task_id}: {e}")

            try:
                file_name = f"{request.task_id or 'unknown_task'}.txt"
                upload_file_from_bytes(
                    filename=file_name,
                    data=content.encode("utf-8"),
                    output_dir=LocalSettings().content_generation_dir,
                )
            except Exception as e:
                print(
                    f"Warning: Failed to upload generated content for task {request.task_id}: {e}"
                )

        return {"content": content}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/review")
async def review_content(request: TextRequest):
    try:
        prompt_task = fetch_clickup_task(task_id="869ak6h10")
        prompt = prompt_task.get("text_content", "") if prompt_task else ""
        style_task = fetch_clickup_task(task_id="869akjejr")
        style = style_task.get("text_content", "") if style_task else ""
        system_prompt = f"{prompt}\n\nStyle Guidelines:\n{style}"
    except Exception as e:
        system_prompt = ""
        print(f"Warning: Failed to fetch system prompt from ClickUp task: {e}")

    try:
        print(f"Chosen model: {request.chat_model}")
        content = get_ollama_content_generation(
            prompt=request.text, model=request.chat_model, system_prompt=system_prompt
        )

        if request.task_id:
            try:
                update_clickup_task(
                    task_id=request.task_id,
                    status="phase 9. review interlink",
                    description=content,
                )
            except Exception as e:
                print(f"Warning: Failed to update ClickUp task {request.task_id}: {e}")

            try:
                file_name = f"{request.task_id or 'unknown_task'}.txt"
                upload_file_from_bytes(
                    filename=file_name,
                    data=content.encode("utf-8"),
                    output_dir=LocalSettings().review_output_dir,
                )
            except Exception as e:
                print(
                    f"Warning: Failed to upload generated content for task {request.task_id}: {e}"
                )

        return {"content": content}
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))